import re, base64, hmac, sha, md5, time, httplib, urlparse, urllib
from xml.dom.minidom import parseString as parseXML

""" Access methods for Amazon S3.

    http://docs.amazonwebservices.com/AmazonS3/2006-03-01/
"""

class Service:

    def __init__(self, access_key, secret_key, service_host='s3.amazonaws.com'):
        self.service_host = service_host
        self.access_key   = access_key
        self.secret_key   = secret_key
    
    
    
    def sendS3ServiceRequest(self, method, path, query='', data='', content_type='text/plain'):
        """ Make a request of Queue REST API.
        """
        date = time.strftime('%a, %d %b %Y %H:%M:%S %Z', time.localtime())
        content_md5 = base64.encodestring(md5.new(data).digest()).strip()
        authorization = 'AWS %s:%s' % (self.access_key, base64.encodestring(hmac.new(self.secret_key, '%(method)s\n%(content_md5)s\n%(content_type)s\n%(date)s\n%(path)s' % locals(), sha).digest()).strip())
    
        headers = {'Date': date,
                   'Content-Type': content_type, 'Content-MD5': content_md5,
                   'Content-Length': len(data), 'Authorization': authorization}
        
        while True:
            try:
                conn = httplib.HTTPConnection(self.service_host)
                conn.request(method, path + query, data, headers)
                return conn.getresponse()
            except:
                print 'Failed httplib.request, sleeping for 10 seconds...'
                time.sleep(10)
    
    
    
    def putBucket(self, name):
        """ Add a new bucket.
        """
        return self.sendS3ServiceRequest('PUT', '/'+name).status == 200
                
    
    
    def deleteBucket(self, name):
        """ Delete a bucket.
        """
        return self.sendS3ServiceRequest('DELETE', '/'+name).status == 204
                
    
    
    def getBuckets(self, name=re.compile(r'.*')):
        """ Retrieve a list of queue ID's.
        """
        return [str(BucketName.firstChild.nodeValue)
                for BucketName
                in [Bucket.getElementsByTagName('Name')[0]
                    for Bucket
                    in parseXML(self.sendS3ServiceRequest('GET', '/').read()).getElementsByTagName('Bucket')]
                if name.search(str(BucketName.firstChild.nodeValue))]
                
    
    
    def listBucketObjects(self, bucket_id, prefix='', marker='', max_keys=40):
        """ Retrieve a list of object ID's.
        """
        return [str(ObjectKey.firstChild.nodeValue)
                for ObjectKey
                in [Contents.getElementsByTagName('Key')[0]
                    for Contents
                    in parseXML(self.sendS3ServiceRequest('GET', '/'+bucket_id, '?'+urllib.urlencode({'prefix':prefix, 'marker':marker, 'max-keys':max_keys})).read()).getElementsByTagName('Contents')]]
                
    
    
    def putBucketObject(self, bucket_id, object_id, object_content, object_type='text/plain'):
        """ Put one object into the bucket, return HTTP status code.
        """
        return self.sendS3ServiceRequest('PUT', '/'+bucket_id+'/'+object_id, data=object_content, content_type=object_type).status == 200
                
    
    
    def getBucketObject(self, bucket_id, object_id):
        """ Get one object from the bucket, in HTTPResponse form.
        """
        return self.sendS3ServiceRequest('GET', '/'+bucket_id+'/'+object_id)
                
    
    
    def headBucketObject(self, bucket_id, object_id):
        """ Get information about an object from the bucket, in HTTPResponse form.
        """
        return self.sendS3ServiceRequest('HEAD', '/'+bucket_id+'/'+object_id)
                
    
    
    def deleteBucketObject(self, bucket_id, object_id):
        """ Delete one object from the bucket, return HTTP status code.
        """
        return self.sendS3ServiceRequest('DELETE', '/'+bucket_id+'/'+object_id).status == 204
    
    
    
    
if __name__ == '__main__':

    ss = Service('<Your Access Key Here>', '<Your Secret Key Here>')

    print 'Adding Bucket...'
    print '  ', ss.putBucket('magicalwonderbucket')

    print 'Getting Buckets...'
    bucket_id = ss.getBuckets(re.compile(r'magicalwonderbucket'))[0]
    print '  Bucket ID:', bucket_id
    
    print 'Putting Object...'
    if ss.putBucketObject(bucket_id, 'greeting', 'Hello World'):
        print '  Done'
    else:
        print '  Failed'
        
    print 'Getting Bucket Objects...'
    object_id = ss.listBucketObjects(bucket_id)[0]
    print '  Found:', object_id
    
    print 'Getting Object Info...'
    print ''.join(['  '+': '.join(header)+'\n' for header in ss.headBucketObject(bucket_id, object_id).getheaders()])
    
    print 'Getting Object...'
    print '  Object Size:', len(ss.getBucketObject(bucket_id, object_id).read())
    
    print 'Deleting Object...'
    if ss.deleteBucketObject(bucket_id, object_id):
        print '  Done'
    else:
        print '  Failed'

    print 'Deleting Bucket...'
    print '  ', ss.deleteBucket('magicalwonderbucket')
